#include "wm_torture.h"

Display *d = NULL;
char result[4096] = "";

static char *test = "";
static Test tests[] = {
#define HANDLER
#include "wm_tests.h"
#undef HANDLER
};

/* internal functions */
static void
x_init(char *dname)
{
   d = XOpenDisplay(dname);
   if (!d)
     {
	printf("ERROR: cannot open X display!\n");
	exit(-1);
     }
}

static void
x_shutdown(void)
{
   XCloseDisplay(d);
}

static void
help(void)
{
   int i;
   
   printf
     ("USAGE:\n"
      "  -display DISPLAYNAME     Connect to DISPLAYNAME X display\n"
      "  -d       DISPLAYNAME     Connect to DISPLAYNAME X display\n"
      "  -help                    Display this help\n"
      "  -h                       Display this help\n"
      "  -wm      NAME            Set the window manager name to NAME\n"
      "  -v       VERSION         Set the window manager version to VERSION\n"
      "  -t       TEST            Run test named TEST\n"
      "\n"
      "TEST may be one of:\n"
      );
   for (i = 0; i < (sizeof(tests) / sizeof(Test)); i++)
     printf("  %s\n", tests[i].name);
   exit(0);
}

/* utility functions */
double
get_time(void)
{
   struct timeval      timev;
   
   gettimeofday(&timev, NULL);
   return (double)timev.tv_sec + (((double)timev.tv_usec) / 1000000);
}

/* DUH! */
int
main(int argc, char **argv)
{
   char *dname = NULL;
   char *wm_name = "Unknown";
   char *wm_version = "Unknown";
   void (*test_fn) (void) = NULL;
   int i, j;
   
   if (argc <= 1)
     help();

   dname = getenv("DISPLAY");
   for (i = 1; i < argc; i++)
     {
	if (((!strcmp(argv[i], "-display")) ||
	     (!strcmp(argv[i], "-d"))) && (i < (argc - 1)))
	  {
	     dname = argv[i + 1];
	     i++;
	  }
	else if ((!strcmp(argv[i], "-help")) ||
		 (!strcmp(argv[i], "-h")))
	  {
	     help();
	  }
	else if ((!strcmp(argv[i], "-wm")) && (i < (argc - 1)))
	  {
	     wm_name = argv[i + 1];
	     i++;
	  }
	else if ((!strcmp(argv[i], "-v")) && (i < (argc - 1)))
	  {
	     wm_version = argv[i + 1];
	     i++;
	  }
	else if ((!strcmp(argv[i], "-t")) && (i < (argc - 1)))
	  {
	     for (j = 0; j < (sizeof(tests) / sizeof(Test)); j++)
	       {
		  if (!strcmp(argv[i + 1], tests[j].name))
		    {
		       test_fn = tests[j].func;
		       test = tests[j].name;
		       break;
		    }
	       }
	     i++;
	  }
     }

   if (!test_fn)
     {
	printf("ERROR: no test selected\n");
	help();
     }
   
   x_init(dname);
   test_fn();
   x_shutdown();

   printf("Results for: %s - %s\n"
	  "Test:        %s\n"
	  "             %s\n",
	  wm_name, wm_version, test, result);
   return 0;
}
